@@ -17,7 +17,7 @@
 #define _POSIX_SOURCE /* feature test macro for fileno */
 #define _XOPEN_SOURCE /* feature test macro for fsync */
 
-#include <tilda-config.h>
+#include "config.h"
 #include "debug.h"
 
 #include <confuse.h>
@@ -81,6 +81,7 @@ static cfg_opt_t config_opts[] = {
     CFG_INT("tab_pos", 0, CFGF_NONE),
     CFG_BOOL("expand_tabs", FALSE, CFGF_NONE),
     CFG_BOOL("show_single_tab", FALSE, CFGF_NONE),
+    CFG_BOOL("insert_tab_after_current", FALSE, CFGF_NONE),
     CFG_INT("backspace_key", 0, CFGF_NONE),
     CFG_INT("delete_key", 1, CFGF_NONE),
     CFG_INT("d_set_title", 3, CFGF_NONE),
@@ -172,9 +173,26 @@ static cfg_opt_t config_opts[] = {
 
     CFG_INT("back_alpha", 0xffff, CFGF_NONE),
 
+    /* Whether bold text is shown as bright colors */
+    CFG_BOOL("bold_is_bright", FALSE, CFGF_NONE),
+
     /* Whether to show the full tab title as a tooltip */
     CFG_BOOL("show_title_tooltip", FALSE, CFGF_NONE),
 
+    /* Whether match activation with mouse click requires CTRL to be pressed */
+    CFG_BOOL("control_activates_match", TRUE, CFGF_NONE),
+
+    /* Whether to enable regular expressions to match
+     * certain types of tokens: */
+    CFG_BOOL("match_web_uris", TRUE, CFGF_NONE),
+    CFG_BOOL("match_file_uris", TRUE, CFGF_NONE),
+    CFG_BOOL("match_email_addresses", TRUE, CFGF_NONE),
+    CFG_BOOL("match_numbers", TRUE, CFGF_NONE),
+
+    /* if set to TRUE, tilda will fall back to open
+     * URIs with the 'web_browser' option. */
+    CFG_BOOL("use_custom_web_browser", FALSE, CFGF_NONE),
+
     /**
      * Deprecated tilda options. These options be commented out in the
      * configuration file and will not be initialized with default values
@@ -257,22 +275,6 @@ gint config_setnint(const gchar *key, const glong val, const guint idx)
     return 0;
 }
 
-gint config_setdouble (const gchar *key, const gdouble val) {
-    config_mutex_lock ();
-    cfg_setfloat (tc, key, val);
-    config_mutex_unlock ();
-
-    return 0;
-}
-
-gint config_setndouble (const gchar *key, const gdouble val, const guint idx) {
-    config_mutex_lock ();
-    cfg_setnfloat (tc, key, val, idx);
-    config_mutex_unlock ();
-
-    return 0;
-}
-
 gint config_setstr (const gchar *key, const gchar *val)
 {
     config_mutex_lock ();
@@ -313,26 +315,6 @@ glong config_getnint(const gchar *key, const guint idx)
     return temp;
 }
 
-gdouble config_getdouble (const gchar* key) {
-    gdouble temp;
-
-    config_mutex_lock ();
-    temp = cfg_getfloat (tc, key);
-    config_mutex_unlock ();
-
-    return temp;
-}
-
-gdouble config_getndouble (const gchar* key, const guint idx) {
-    gdouble temp;
-
-    config_mutex_lock ();
-    temp = cfg_getnfloat (tc, key, idx);
-    config_mutex_unlock ();
-
-    return temp;
-}
-
 gchar* config_getstr (const gchar *key)
 {
     gchar *temp;
