@@ -13,7 +13,7 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <tilda-config.h>
+#include "config.h"
 
 #include "debug.h"
 #include "tilda.h"
@@ -21,7 +21,6 @@
 #include "tilda_window.h"
 #include "tilda_terminal.h"
 #include "key_grabber.h"
-#include "vte-util.h"
 
 #include <math.h>
 #include <stdio.h>
@@ -197,6 +196,16 @@ gint toggle_fullscreen_cb (tilda_window *tw)
     return GDK_EVENT_STOP;
 }
 
+void tilda_window_set_dbus_enabled (tilda_window *tw, gboolean enabled)
+{
+    tw->dbus_enabled = enabled;
+}
+
+gboolean tilda_window_get_dbus_enabled (tilda_window *tw)
+{
+    return tw->dbus_enabled;
+}
+
 static gint toggle_transparency_cb (tilda_window *tw)
 {
     DEBUG_FUNCTION ("toggle_transparency");
@@ -235,30 +244,6 @@ static void tilda_window_apply_transparency (tilda_window *tw, gboolean status)
         }
 }
 
-static gboolean
-search_gregex_cb (TildaSearchBox       *search,
-                  GRegex               *regex,
-                  TildaSearchDirection  direction,
-                  gboolean              wrap_on_search,
-                  tilda_window         *tw)
-{
-    VteTerminal *vte_terminal;
-    tilda_term *term;
-
-    term = tilda_window_get_current_terminal (tw);
-
-    vte_terminal = VTE_TERMINAL (term->vte_term);
-
-    vte_terminal_search_set_gregex (vte_terminal, regex, (GRegexMatchFlags) 0);
-
-    vte_terminal_search_set_wrap_around (vte_terminal, wrap_on_search);
-
-    if (direction == SEARCH_BACKWARD)
-        return vte_terminal_search_find_previous (vte_terminal);
-    else
-        return vte_terminal_search_find_next (vte_terminal);
-}
-
 static gboolean
 search_cb (TildaSearchBox       *search,
            VteRegex             *regex,
@@ -691,7 +676,12 @@ static gint ccopy (tilda_window *tw)
         list = list->next;
     }
 
+    if ( ! vte_terminal_get_has_selection (VTE_TERMINAL (list->data)) ) {
+        return GDK_EVENT_PROPAGATE;
+    }
+
     vte_terminal_copy_clipboard_format (VTE_TERMINAL(list->data), VTE_FORMAT_TEXT);
+    vte_terminal_unselect_all (VTE_TERMINAL(list->data));
 
     return GDK_EVENT_STOP;
 }
@@ -723,6 +713,8 @@ static gint tilda_add_config_accelerator_by_path(const gchar* key, const gchar*
     GdkModifierType accel_mods;
     GClosure *temp;
 
+    if (strcmp(key, "NULL") == 0) return TRUE;
+
     gtk_accelerator_parse (config_getstr(key), &accel_key, &accel_mods);
     if (! ((accel_key == 0) && (accel_mods == 0)) )  // make sure it parsed properly
     {
@@ -742,7 +734,7 @@ gboolean tilda_window_update_keyboard_accelerators (const gchar* path, const gch
     return gtk_accel_map_change_entry(path, accel_key, accel_mods, FALSE);
 }
 
-/* This function does the setup of the keyboard acceleratos. It should only be called once when the tilda window is
+/* This function does the setup of the keyboard accelerators. It should only be called once when the tilda window is
  * initialized. Use tilda_window_update_keyboard_accelerators to update keybindings that have been changed by the user.
  */
 static gint tilda_window_setup_keyboard_accelerators (tilda_window *tw)
@@ -956,6 +948,9 @@ gboolean tilda_window_init (const gchar *config_file, const gint instance, tilda
     /* Create the notebook */
     tw->notebook = gtk_notebook_new ();
 
+    /* Adding widget title for CSS selection */
+    gtk_widget_set_name (GTK_WIDGET(tw->window), "Main");
+
     /* Here we setup the CSS settings for the GtkNotebook.
      * If the option "Show notebook border" in the preferences is not
      * checked. Then we disable the border. Otherwise nothing is changed.
@@ -1029,17 +1024,16 @@ gboolean tilda_window_init (const gchar *config_file, const gint instance, tilda
     GtkWidget *main_box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
     tw->search = tilda_search_box_new ();
 
+    GtkStyleContext *context = gtk_widget_get_style_context(main_box);
+    gtk_style_context_add_class(context, GTK_STYLE_CLASS_BACKGROUND);
+
     gtk_container_add (GTK_CONTAINER(tw->window), main_box);
     gtk_box_pack_start (GTK_BOX (main_box), tw->notebook, TRUE, TRUE, 0);
     gtk_box_pack_start (GTK_BOX (main_box), tw->search, FALSE, TRUE, 0);
 
-    if (VTE_CHECK_VERSION_RUMTIME (0, 56, 1)) {
-        g_signal_connect (tw->search, "search",
-                          G_CALLBACK (search_cb), tw);
-    } else {
-        g_signal_connect (tw->search, "search-gregex",
-                          G_CALLBACK (search_gregex_cb), tw);
-    }
+    g_signal_connect (tw->search, "search",
+                      G_CALLBACK (search_cb), tw);
+
     g_signal_connect (tw->search, "focus-out",
                       G_CALLBACK (search_focus_out_cb), tw);
 
@@ -1124,7 +1118,14 @@ gint tilda_window_add_tab (tilda_window *tw)
     GtkWidget *label;
     gint index;
 
-    tt = tilda_term_init (tw);
+    /* Determine where to insert the new terminal */
+    index = -1;
+    if (config_getbool ("insert_tab_after_current")) {
+        index = 1 + gtk_notebook_get_current_page (GTK_NOTEBOOK(tw->notebook));
+    }
+
+    /* Initialize the terminal */
+    tt = tilda_term_init (tw, index);
 
     if (tt == NULL)
     {
@@ -1133,10 +1134,9 @@ gint tilda_window_add_tab (tilda_window *tw)
         return FALSE;
     }
 
-    /* Create page and append to notebook */
+    /* Create page and insert it into the notebook */
     label = gtk_label_new (config_getstr("title"));
-    /* Strangely enough, prepend puts pages on the end */
-    index = gtk_notebook_append_page (GTK_NOTEBOOK(tw->notebook), tt->hbox, label);
+    index = gtk_notebook_insert_page (GTK_NOTEBOOK(tw->notebook), tt->hbox, label, index);
     gtk_notebook_set_current_page (GTK_NOTEBOOK(tw->notebook), index);
     gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK(tw->notebook), tt->hbox, TRUE);
 
