@@ -75,7 +75,10 @@ struct tilda_window_
     /* Should Tilda hide itself even if not focused */
     gboolean hide_non_focused;
 
-	gboolean fullscreen;
+    gboolean fullscreen;
+
+    /* If tilda was started with D-Bus support */
+    gboolean dbus_enabled;
 
     /* This field MUST be set before calling pull()! */
     enum tilda_animation_state current_state;
@@ -142,8 +145,8 @@ gint tilda_window_prev_tab (tilda_window *tw);
 /**
  * tilda_window_init ()
  *
- * Initalizes an already allocated tilda_window *. It will also initialize and set up
- * as much of the window as possible using the values in the configuation system.
+ * Initializes an already allocated tilda_window *. It will also initialize and set up
+ * as much of the window as possible using the values in the configuration system.
  *
  * @param instance the instance number of this tilda_window
  *
@@ -168,6 +171,20 @@ void tilda_window_set_fullscreen(tilda_window *tw);
  */
 gint toggle_fullscreen_cb (tilda_window *tw);
 
+/**
+ * Set the D-Bus state in the tilda window, to indicate if D-Bus is enabled.
+ * @param tw An instance of a tilda_window struct.
+ * @param enabled TRUE if D-Bus is enabled for this instance, false otherwise.
+ */
+void tilda_window_set_dbus_enabled (tilda_window *tw, gboolean enabled);
+
+/**
+ * Get the D-Bus state in the tilda window.
+ * @param tw An instance of a tilda_window struct.q
+ * @return Returns TRUE if D-Bus is enabled, FALSE otherwise.
+ */
+gboolean tilda_window_get_dbus_enabled (tilda_window *tw);
+
 /**
  * This controls where the tabs are positions (e.g. top, left, bottom or
  * right).
