@@ -14,7 +14,7 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <tilda-config.h>
+#include "config.h"
 
 #include <errno.h>
 
@@ -113,7 +113,8 @@ gint wizard (tilda_window *tw)
     /* Copy the current program state into the wizard */
     set_wizard_state_from_config (tw);
 
-    wizard->keybinding = tilda_keybinding_init (wizard->builder);
+    wizard->keybinding = tilda_keybinding_init (wizard->builder,
+                                                tilda_window_get_dbus_enabled (tw));
 
     /* Connect all signal handlers. We do this after copying the state into
      * the wizard so that all of the handlers don't get called as we copy in
@@ -740,6 +741,90 @@ static void check_command_login_shell_cb (GtkWidget *w, tilda_window *tw) {
     config_setbool("command_login_shell", active);
 }
 
+static void check_control_activates_match_cb (GtkWidget *w, tilda_window *tw) {
+    const gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
+
+    config_setbool("control_activates_match", active);
+}
+
+static void update_custom_web_browser_sensitivity () {
+    gboolean match_web_uris = config_getbool("match_web_uris");
+    gboolean use_custom_web_browser = config_getbool("use_custom_web_browser");
+    gboolean sensitive = FALSE;
+
+    GtkWidget * check_custom_web_browser;
+    GtkWidget * label_web_browser;
+    GtkWidget * entry_web_browser;
+
+    check_custom_web_browser =
+            GTK_WIDGET (gtk_builder_get_object (xml, "check_custom_web_browser"));
+    label_web_browser =
+            GTK_WIDGET (gtk_builder_get_object (xml, "label_web_browser"));
+    entry_web_browser =
+            GTK_WIDGET (gtk_builder_get_object (xml, "entry_web_browser"));
+
+    gtk_widget_set_sensitive(check_custom_web_browser, match_web_uris);
+
+    sensitive = match_web_uris && use_custom_web_browser;
+
+    gtk_widget_set_sensitive (label_web_browser, sensitive);
+    gtk_widget_set_sensitive (entry_web_browser, sensitive);
+}
+
+static void check_match_web_uris_cb (GtkWidget *w, tilda_window *tw) {
+    const gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
+
+    config_setbool("match_web_uris", active);
+
+    update_custom_web_browser_sensitivity ();
+
+    for (guint i=0; i < g_list_length (tw->terms); i++) {
+        tilda_term * tt = g_list_nth_data (tw->terms, i);
+        tilda_terminal_update_matches(tt);
+    }
+}
+
+static void check_custom_web_browser_cb (GtkWidget *w, tilda_window *tw) {
+    const gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
+
+    config_setbool("use_custom_web_browser", active);
+
+    update_custom_web_browser_sensitivity ();
+}
+
+static void check_match_file_uris_cb (GtkWidget *w, tilda_window *tw) {
+    const gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
+
+    config_setbool("match_file_uris", active);
+
+    for (guint i=0; i < g_list_length (tw->terms); i++) {
+        tilda_term * tt = g_list_nth_data (tw->terms, i);
+        tilda_terminal_update_matches(tt);
+    }
+}
+
+static void check_match_email_addresses_cb (GtkWidget *w, tilda_window *tw) {
+    const gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
+
+    config_setbool("match_email_addresses", active);
+
+    for (guint i=0; i < g_list_length (tw->terms); i++) {
+        tilda_term * tt = g_list_nth_data (tw->terms, i);
+        tilda_terminal_update_matches(tt);
+    }
+}
+
+static void check_match_numbers_cb (GtkWidget *w, tilda_window *tw) {
+    const gboolean active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
+
+    config_setbool("match_numbers", active);
+
+    for (guint i=0; i < g_list_length (tw->terms); i++) {
+        tilda_term * tt = g_list_nth_data (tw->terms, i);
+        tilda_terminal_update_matches(tt);
+    }
+}
+
 static void check_start_fullscreen_cb(GtkWidget *w, tilda_window *tw) {
     const gboolean start_fullscreen = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
 
@@ -768,6 +853,13 @@ static void check_show_title_tooltip_toggled_cb (GtkWidget *w, tilda_window *tw)
     window_title_change_all (tw);
 }
 
+static void check_insert_tab_after_current_toggled_cb (GtkWidget *w, tilda_window *tw)
+{
+    const gboolean insert_tab_after_current = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
+
+    config_setbool("insert_tab_after_current", insert_tab_after_current);
+}
+
 static void entry_web_browser_changed (GtkWidget *w, tilda_window *tw) {
     const gchar *web_browser = gtk_entry_get_text (GTK_ENTRY(w));
 
@@ -1483,6 +1575,20 @@ static void colorbutton_palette_n_set_cb (GtkWidget *w, tilda_window *tw)
     }
 }
 
+static void check_bold_is_bright_toggled_cb (GtkWidget *w, tilda_window *tw)
+{
+    const gboolean status = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w));
+    guint i;
+    tilda_term *tt;
+
+    config_setbool ("bold_is_bright", status);
+
+    for (i=0; i<g_list_length (tw->terms); i++) {
+        tt = g_list_nth_data (tw->terms, i);
+        vte_terminal_set_bold_is_bright (VTE_TERMINAL(tt->vte_term), status);
+    }
+}
+
 static void combo_scrollbar_position_changed_cb (GtkWidget *w, tilda_window *tw)
 {
     const gint status = gtk_combo_box_get_active (GTK_COMBO_BOX(w));
@@ -1717,7 +1823,7 @@ static void initialize_geometry_spinners(tilda_window *tw) {
     gint screen_width, screen_height;
     screen_size_get_dimensions (&screen_width, &screen_height);
     SPIN_BUTTON_SET_RANGE("spin_x_position", 0, screen_width);
-    SPIN_BUTTON_SET_VALUE("spin_x_position", xpos); /* TODO: Consider x in rectange.x for monitor displacement */
+    SPIN_BUTTON_SET_VALUE("spin_x_position", xpos); /* TODO: Consider x in rectangle.x for monitor displacement */
 
     gint ypos = config_getint("y_pos");
     if(ypos < rectangle.y) {
@@ -1772,6 +1878,16 @@ static void set_wizard_state_from_config (tilda_window *tw) {
     CHECK_BUTTON ("check_run_custom_command", "run_command");
     TEXT_ENTRY ("entry_custom_command", "command");
     CHECK_BUTTON ("check_command_login_shell", "command_login_shell");
+
+    CHECK_BUTTON ("check_control_activates_match", "control_activates_match");
+
+    CHECK_BUTTON ("check_match_web_uris", "match_web_uris");
+    CHECK_BUTTON ("check_custom_web_browser", "use_custom_web_browser");
+
+    CHECK_BUTTON ("check_match_file_uris", "match_file_uris");
+    CHECK_BUTTON ("check_match_email_addresses", "match_email_addresses");
+    CHECK_BUTTON ("check_match_numbers", "match_numbers");
+
     COMBO_BOX ("combo_command_exit", "command_exit");
     COMBO_BOX ("combo_on_last_terminal_exit", "on_last_terminal_exit");
     CHECK_BUTTON ("check_prompt_on_exit", "prompt_on_exit");
@@ -1780,6 +1896,8 @@ static void set_wizard_state_from_config (tilda_window *tw) {
 
     TEXT_ENTRY ("entry_web_browser", "web_browser");
 
+    update_custom_web_browser_sensitivity ();
+
     CHECK_BUTTON ("check_confirm_close_tab", "confirm_close_tab");
 
     /* Appearance Tab */
@@ -1797,6 +1915,7 @@ static void set_wizard_state_from_config (tilda_window *tw) {
     CHECK_BUTTON ("check_expand_tabs", "expand_tabs");
     CHECK_BUTTON ("check_show_single_tab", "show_single_tab");
     CHECK_BUTTON ("check_show_title_tooltip", "show_title_tooltip");
+    CHECK_BUTTON ("check_insert_tab_after_current", "insert_tab_after_current");
 
     SET_SENSITIVE_BY_CONFIG_BOOL ("label_level_of_transparency","enable_transparency");
     SET_SENSITIVE_BY_CONFIG_BOOL ("spin_level_of_transparency","enable_transparency");
@@ -1836,6 +1955,8 @@ static void set_wizard_state_from_config (tilda_window *tw) {
         update_palette_color_button(i);
     }
 
+    CHECK_BUTTON ("check_bold_is_bright", "bold_is_bright");
+
     /* Scrolling Tab */
     initialize_scrollback_settings();
 
@@ -1930,6 +2051,15 @@ static void connect_wizard_signals (TildaWizard *wizard)
     CONNECT_SIGNAL ("combo_command_exit","changed",combo_command_exit_changed_cb, tw);
     CONNECT_SIGNAL ("check_command_login_shell", "toggled", check_command_login_shell_cb, tw);
 
+    CONNECT_SIGNAL ("check_control_activates_match", "toggled", check_control_activates_match_cb, tw);
+
+    CONNECT_SIGNAL ("check_match_web_uris", "toggled", check_match_web_uris_cb, tw);
+    CONNECT_SIGNAL ("check_custom_web_browser", "toggled", check_custom_web_browser_cb, tw);
+
+    CONNECT_SIGNAL ("check_match_file_uris", "toggled", check_match_file_uris_cb, tw);
+    CONNECT_SIGNAL ("check_match_email_addresses", "toggled", check_match_email_addresses_cb, tw);
+    CONNECT_SIGNAL ("check_match_numbers", "toggled", check_match_numbers_cb, tw);
+
     CONNECT_SIGNAL ("entry_web_browser","changed",entry_web_browser_changed, tw);
     CONNECT_SIGNAL ("entry_web_browser","focus-out-event", validate_executable_command_cb, tw);
 
@@ -1951,6 +2081,7 @@ static void connect_wizard_signals (TildaWizard *wizard)
     CONNECT_SIGNAL ("check_expand_tabs","toggled",check_expand_tabs_toggled_cb, tw);
     CONNECT_SIGNAL ("check_show_single_tab","toggled",check_show_single_tab_toggled_cb, tw);
     CONNECT_SIGNAL ("check_show_title_tooltip","toggled",check_show_title_tooltip_toggled_cb, tw);
+    CONNECT_SIGNAL ("check_insert_tab_after_current","toggled",check_insert_tab_after_current_toggled_cb, tw);
 
     CONNECT_SIGNAL ("check_enable_transparency","toggled",check_enable_transparency_toggled_cb, tw);
     CONNECT_SIGNAL ("check_animated_pulldown","toggled",check_animated_pulldown_toggled_cb, tw);
@@ -1970,6 +2101,7 @@ static void connect_wizard_signals (TildaWizard *wizard)
         CONNECT_SIGNAL (s,"color-set",colorbutton_palette_n_set_cb, tw);
         g_free (s);
     }
+    CONNECT_SIGNAL ("check_bold_is_bright","toggled",check_bold_is_bright_toggled_cb, tw);
 
     /* Scrolling Tab */
     CONNECT_SIGNAL ("combo_scrollbar_position","changed",combo_scrollbar_position_changed_cb, tw);
